package de.hft.stuttgart.citydoctor2.datastructure;

import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.CityGmlUtils;
import org.citygml4j.core.model.transportation.AuxiliaryTrafficArea;
import org.citygml4j.core.model.transportation.TrafficArea;
import org.citygml4j.core.util.geometry.GeometryFactory;
import org.xmlobjects.gml.model.geometry.aggregates.MultiSurface;
import org.xmlobjects.gml.model.geometry.aggregates.MultiSurfaceProperty;


public class TrafficAreaObject extends TransportationObject {

    public enum TrafficAreaType {
        TRAFFIC_AREA, AUXILIARY_TRAFFIC_AREA
    }

    public TrafficAreaObject(TrafficAreaType trafficAreaType) {
        if (trafficAreaType == TrafficAreaType.TRAFFIC_AREA) {
            setType(TransportationType.TRAFFIC_AREA);
        } else {
            setType(TransportationType.AUXILLIARY_TRAFFIC_AREA);
        }
    }

    @Override
    public void reCreateGeometries(GeometryFactory factory, ParserConfiguration config) {
        for (Geometry geom : getGeometries()) {
            if (geom instanceof ImplicitGeometryHolder) {
                continue;
            }
            if (geom.getType() == GeometryType.MULTI_SURFACE) {
                MultiSurface ms = CityGmlUtils.createMultiSurface(geom, factory, config);
                switch (type) {
                    case TRAFFIC_AREA:
                        TrafficArea ta = (TrafficArea) super.getGmlObject();
                        setMultiSurfaceAccordingToLod(ta, ms, geom.getLod());
                        break;
                    case AUXILLIARY_TRAFFIC_AREA:
                        AuxiliaryTrafficArea ata = (AuxiliaryTrafficArea) super.getGmlObject();
                        setMultiSurfaceAccordingToLod(ata, ms, geom.getLod());
                        break;
                }
            } else {
                throw new IllegalStateException("Geometry in TransportationObject cannot be of type " + geom.getType()
                        + ". Only MultiSurface allowed");
            }
        }
    }


    private void setMultiSurfaceAccordingToLod(AuxiliaryTrafficArea ata, MultiSurface ms, Lod lod) {
        switch (lod) {
            case LOD0:
                ata.setLod0MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD1:
                ata.setLod1MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD2:
                ata.setLod2MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD3:
                ata.setLod3MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD4:
                ata.getDeprecatedProperties().setLod4MultiSurface(new MultiSurfaceProperty(ms));
                break;
            default:
                throw new IllegalStateException("cannot set geometry with LOD for AuxiliaryTrafficArea: " + lod);
        }
    }

    private void setMultiSurfaceAccordingToLod(TrafficArea ta, MultiSurface ms, Lod lod) {
        switch (lod) {
            case LOD0:
                ta.setLod0MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD1:
                ta.setLod1MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD2:
                ta.setLod2MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD3:
                ta.setLod3MultiSurface(new MultiSurfaceProperty(ms));
                break;
            case LOD4:
                ta.getDeprecatedProperties().setLod4MultiSurface(new MultiSurfaceProperty(ms));
                break;
            default:
                throw new IllegalStateException("cannot set geometry with LOD for TrafficArea: " + lod);
        }
    }


    @Override
    public void unsetGmlGeometries() {
        switch (type) {
            case TRAFFIC_AREA:
                TrafficArea ta = (TrafficArea) super.getGmlObject();
                ta.setLod2MultiSurface(null);
                ta.setLod3MultiSurface(null);
                ta.getDeprecatedProperties().setLod4MultiSurface(null);
                break;
            case AUXILLIARY_TRAFFIC_AREA:
                AuxiliaryTrafficArea ata = (AuxiliaryTrafficArea) super.getGmlObject();
                ata.setLod0MultiSurface(null);
                ata.setLod1MultiSurface(null);
                ata.setLod2MultiSurface(null);
                ata.setLod3MultiSurface(null);
                ata.getDeprecatedProperties().setLod4MultiSurface(null);
                break;
        }
    }

    @Override
    public CityObject getTopLevelCityObject() {
        return this;
    }

    @Override
    public String toString() {
        return "TrafficArea [id=" + getGmlId() + "]";
    }


}
